/* ldistfp - linux distribution fingerprinting
 *
 * by scut / teso
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "network.h"

#define	AUTHORS	"-sc/teso"
#define	VERSION	"0.1.3"


typedef struct	fingerprint {
	char *	distname;
	char *	distversion;
	char *	substring;
	char *	auth_version;
	int	exact;
} fingerprint;

fingerprint **	fp_list = NULL;
char *		fp_filename = "ldistfp-auth-fingerprints";
int		fp_count = 0;

int		strictsure = 0;
int		machineoutput = 0;
FILE *		newfile = NULL;	/* file to save new fingerprints to */


void	usage (char *prog);
void	host_print (char *host, char *bogus, fingerprint *fp);
void	fp_det (char *host, char *buf);
void	fp_list_read (char *filename);


void
usage (char *prog)
{
	fprintf (stderr, "usage: %s [-r] [-n <outfile>] [-s] [-m] <host|ip>\n\n", prog);
	fprintf (stderr, "\t-r             rawmode, read from stdin in format [^ ] (buf$)\n");
	fprintf (stderr, "\t-n <outfile>   output new fingerprint information to this file\n");
	fprintf (stderr, "\t-s             only print host information if properly identified\n");
	fprintf (stderr, "\t-m             machine output\n\n");

	exit (EXIT_FAILURE);
}

int
main (int argc, char *argv[])
{
	int	sock,
		n;
	int	rawmode = 0;
	char	c;
	char *	rbuf;
	char *	target;


	fprintf (stderr, "ldistfp "VERSION" - remote identd fingerprinting tool. "AUTHORS"\n\n");

	if (argc < 2)
		usage (argv[0]);

	while ((c = getopt (argc, argv, "rn:sm")) != EOF) {
		switch (c) {
		case 'r':
			rawmode = 1;
			break;
		case 'n':
			newfile = fopen (optarg, "a");
			if (newfile == NULL) {
				perror ("fopen");
				exit (EXIT_FAILURE);
			}
			break;
		case 's':
			strictsure = 1;
			break;
		case 'm':
			machineoutput = 1;
			break;
		default:
			usage ("ldistfp");
			break;
		}
	}

	target = argv[argc - 1];
	if (target[0] == '-')
		usage ("ldistfp");

	fp_list_read (fp_filename);
	fprintf (stderr, "read %d fingerprints\n", fp_count);

	if (rawmode) {
		unsigned char	tgt[200];
		unsigned char	ibuf[1024];

		while (fgets (ibuf, sizeof (ibuf), stdin) != NULL) {
			memset (tgt, '\0', sizeof (tgt));
			sscanf (ibuf, "%199[^ :]", tgt);
			tgt[sizeof (tgt) - 1] = '\0';
			fp_det (tgt, ibuf);
		}
	} else {

		sock = net_connect (NULL, target, 113, NULL, 0, 30);
		if (sock == -1) {
			perror ("net_connect");
			exit (EXIT_FAILURE);
		}

		net_write (sock, "VERSION\n");
		n = net_rlineta (sock, &rbuf, 15);
		if (n < 0 && strictsure == 0) {
			printf ("%s: failed to determine remote version\n", argv[1]);
		} else {
			fp_det (target, rbuf);
		}

		fprintf (stderr, "\n");
		close (sock);
	}

	if (newfile != NULL)
		fclose (newfile);


	exit (EXIT_SUCCESS);
}


void
fp_det (char *host, char *buf)
{
	int	i;

	for (i = 0 ; fp_list[i] != NULL ; ++i) {
		if (strstr (buf, fp_list[i]->substring) != NULL) {

			/* if it is an exact hit, then we print it and return,
			 * since there cannot be any other hit
			 */
			if (fp_list[i]->exact != 0) {
				host_print (host, "", fp_list[i]);
				return;
			} else if (strictsure == 0) {
				host_print (host, "possible", fp_list[i]);
			}
		}
	}

	if (machineoutput == 0 && newfile == NULL) {
		printf ("\nunknown, if you know it write down the following line and mail it\n"
			"to scut@nb.in-berlin.de, thanks.\n\n%s\n", buf);
		return;
	}

	if (newfile == NULL)
		return;

	/* write new fingerprint to newfile */
	fprintf (newfile, "%s: %s", host, buf);

	return;
}


void
host_print (char *host, char *bogus, fingerprint *fp)
{
	if (machineoutput == 1) {
		printf ("%s/%s/%s/%s/%s\n",
		host,
		bogus,
		fp->distname,
		fp->distversion,
		fp->auth_version);
	} else {
		printf ("%s: %s%s %s running %s\n",
		host,
		bogus,
		fp->distname,
		fp->distversion,
		fp->auth_version);
	}

	return;
}

void
fp_list_read (char *filename)
{
	int	n;
	int	fpc = 1;
	FILE *	fpl;

	fpl = fopen (filename, "r");
	if (fpl == NULL) {
		perror ("finger print file");
		exit (EXIT_FAILURE);
	}

	do {
		fingerprint *	new = xcalloc (1, sizeof (fingerprint));

		fpc += 1;
		fp_list = xrealloc (fp_list, fpc * sizeof (fingerprint *));
		fp_list[fpc - 1] = NULL;
		fp_list[fpc - 2] = new;

		new->distname = xcalloc (1, 128);
		new->distversion = xcalloc (1, 128);
		new->substring = xcalloc (1, 128);
		new->auth_version = xcalloc (1, 128);

		/* format: "substring" "dist name" "dist version" "identd version" 0|1
		 */
		do {
			unsigned char	buf[256];

			fgets (buf, sizeof (buf), fpl);
			n = 1;
			if (strlen (buf) > 1 && buf[0] != '#') {
				n = fscanf (fpl, "\"%127[^\"]\"%*[\t ]\"%127[^\"]\"%*[\t ]"
					"\"%127[^\"]\"%*[\t ]\"%127[^\"]\"%*[\t ]%d\n",
					new->distname, new->distversion, new->substring,
					new->auth_version, &new->exact);
			}
		} while (n != EOF && n != 5);

		if (n == EOF) {
			free (new->distname);
			free (new->distversion);
			free (new->substring);
			free (new->auth_version);
			free (new);
			fp_list[fpc - 2] = NULL;
		} else {
			fp_count++;
		}

		if (n != 5)
			n = EOF;

	} while (n != EOF);

	fclose (fpl);
}


