/*
 * THE NEMESIS PROJECT
 * Copyright (C) 1999, 2000, 2001 Mark Grimes <obecian@packetninja.net>
 * Portions copyright (C) 2001 Jeff Nathan <jeff@wwti.com>
 *
 * nemesis-proto_arp.c (ARP Packet Generator)
 *
 */

#include "nemesis-arp.h"

int
buildarp()
{
   int             n;
   static u_char  *pkt;
   static int      sockfd = -1;
   pcap_t         *temp_pcap_d = NULL;

   link_type = 0, pcap_offset = 0;

   /* sanity checks */
   if (payload == NULL)
      payload_s = 0;
   else
      payload_s = strlen(payload);

   if ((temp_pcap_d = pcap_open_live(device, 0, 0, 0, errbuf)) == NULL) {
      printf("Error opening interface: %s\n", errbuf);
      exit(-1);
   }
   link_type = pcap_datalink(temp_pcap_d);
   pcap_close(temp_pcap_d);

   /* verify supported link layer - from bane by dreck */
   
   switch (link_type) { /* Ethernet? Set offset and open l2w handle */
   case DLT_EN10MB:
      pcap_offset = 14;
      if ((l2 = libnet_open_link_interface(device, errbuf)) == NULL) {
         printf("Error Opening layer2 dev for writing: %s\n", errbuf);
         exit(-1);
      }
      break;
   /* Other? Set offset and open l3w handle */
   case DLT_NULL:
   case DLT_PPP:
      pcap_offset = 0;
      if ((l3 = libnet_open_raw_sock(IPPROTO_RAW)) == -1) {
         perror("Error Opening layer3 sock for writing:");
         exit(-1);
      }
      break;
   default:
      printf("Sorry, %s does not use a supported datalink type.\n", device);
      exit(0);
      break;
   }

   if (libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H + payload_s, &pkt) == -1) {
      printf("libnet_init_packet memory error");
      exit(-1);
   }

   libnet_build_ethernet(enet_dst,
			 enet_src,
			 (ltype == 0 ? ETHERTYPE_ARP : ETHERTYPE_REVARP),
			 NULL,
			 0,
			 pkt);

   if (reply == 0) {		/* 0 = ask (request) */
      libnet_build_arp(ARPHRD_ETHER,	/* hardware address type */
		       ETHERTYPE_IP,	/* protocol address type */
		       6,		/* hardware address length */
		       4,		/* protocol address length */
		       (ltype == 0 ? ARPOP_REQUEST : ARPOP_REVREQUEST),	
							/* opcode command */
                       (arp_src_enable == 0 ? enet_src : arp_src),
							/* sender hardware 
							* address */
		       (u_char *) & source,	/* sender protocol (IP) 
						* address */
		       arp_dst,		/* target hardware address */
		       (u_char *) & dest,	/* destination protocol (IP)
						* address */
		       payload,		/* ARP payload pointer */
		       payload_s,	/* ARP payload size */
		       pkt + LIBNET_ETH_H);	/* packet header memory */
   } else if (reply == 1) {	/* 1 = tell (reply) */
      libnet_build_arp(ARPHRD_ETHER,	/* hardware address type */
		       ETHERTYPE_IP,	/* protocol address type */
		       6,		/* hardware address length */
		       4,		/* protocol address length */
		       (ltype == 0 ? ARPOP_REPLY : ARPOP_REVREPLY),
							/* opcode command */
		       (arp_src_enable == 0 ? enet_src : arp_src),
							/* sender hardware 
							* address */
		       (u_char *) & source,	/* sender protocol (IP)
						* address */
		       arp_dst,			/* target hardware address */
		       (u_char *) & dest,	/* destination protocol (IP)
						* address */
		       payload,			/* ARP payload pointer */
		       payload_s,		/* ARP payload size */
		       pkt + LIBNET_ETH_H);	/* packet header memory */
   }
   n = libnet_write_link_layer(l2,
			       device,
			       pkt,
			       LIBNET_ARP_H + LIBNET_ETH_H + payload_s);
   
   if (verbose) {
      if ((reply == 0) && (enet_dst[0] != 0xff) && (enet_dst[1] != 0xff) && 
		(enet_dst[2] != 0xff) && (enet_dst[3] != 0xff) && 
		(enet_dst[4] != 0xff) && (enet_dst[5] != 0xff)) {
      printf("Write %d byte *directed* ARP request packet through linktype %d\n", n, l2->linktype);
      } else { 
         printf("Wrote %d byte %s packet through linktype %d\n", n, 
		(ltype == 0 ? "ARP" : "RARP"), l2->linktype);
      }
   }
   libnet_destroy_packet(&pkt);
   libnet_close_raw_sock(sockfd);
   return (n);
}
