#
# perl/config.pl
#
# rewrite saint.cf after the user changes it.
#
# suck in the changes, then just cycle through each line of the .cf file.
# if there is a match, put the new value in there.
#

sub write_config_file {
local($new_values) = @_;
local(%new_values, $variable, $old_variable, $old_value);

#
# split the strings into something easier to handle
for ( split(/\n/, $new_values) ) {
	next if !$_;

	($variable, $value) = split(/=/, $_);

	# need to stick a dollar sign in front of var
	$variable = "\$" . "$variable";
	# and quotes around non-numbers
	if ($value !~ /^\d+$/) { $value = "\"$value\""; }

	$new_values{$variable} = $value;
	}

#
# open the config and the scratch file
#
die "Can't read $SAINT_CF file!\n" unless open(CF, "$SAINT_CF");
die "Can't write $SAINT_CF.new file!\n" unless open(CFN, ">$SAINT_CF.new");

while (<CF>) {
	# punt if the going gets too tough...
	if (!/^\$/) {
		print CFN $_;
		next;
		}
	
	chop;
	($old_variable, $old_value) = split(/\s*=\s*/, $_);

	# kill spaces and semicolons
	$old_variable =~ s/\s//g;
	$old_value =~ s/;//g;

	# suck in the lines, compare them to each of the vars gotten from user
	for $variable (keys %new_values) {
		if ($variable eq $old_variable) {
			$old_value = $new_values{$variable};
			}
		}

	print "CF: $_ ($old_variable, $old_value)\n";
	print CFN "$old_variable = $old_value;\n";
	}

close(CF);
close(CFN);

# move the evidence to where it belongs... old to .old, new to .cf:
system("mv $SAINT_CF $SAINT_CF.old");
system("mv $SAINT_CF.new $SAINT_CF");

}

# rewrite the config file after changing the custom scan
# s.k. 10/00

sub process_custom {

    my($variables) = @_;
    my($i);
    local($tcp_ports, $udp_ports);

    @custom = ();
    for (split /\n/, $variables) {
	if (/(tcp|udp)_ports=(.*)/) {
	    $i = "${1}_ports";
	    ($$i = $2) =~ s/(\s|\n)//g;
	}
	if (/(.*)_(saint|sara)(q?)/) {
	    if ($3) {
	        push @custom, $1 . "." . $2 . "?";
	    } else {
		push @custom, $1 . "." . $2;
	    }
	}
    }
    for (@custom) {
	$_ .= " $tcp_ports" if /tcpscan\.saint/;
	$_ .= " $udp_ports" if /udpscan\.saint/;
    }
    &write_custom();
}

sub write_custom {

    die "Can't read $SAINT_CF file!\n" unless open(CF, "$SAINT_CF");
    die "Can't write $SAINT_CF.new file!\n" unless open(CFN, ">$SAINT_CF.new");

    while(<CF>) {
	if (/\@custom/) {
	    # skip over the old @custom definition
	    until (/\)/)
		{ $_ = <CF>; }
	    # write the new @custom definition
	    print CFN "\@custom = (\n";
	    for (@custom) {
		print CFN "\t\'$_\',\n";
	    }
	    print CFN "\t);\n";
	}
	else {
	    # set the attack level to 5 (custom)
	    s/$1/5/ if /^\$attack_level\s*=\s*(\d+)\s*;/;
	    print CFN;
	}
    }
    close(CF);
    close(CFN);

    system("mv $SAINT_CF $SAINT_CF.old");
    system("mv $SAINT_CF.new $SAINT_CF");

    # now set the custom scan level in memory
    $attack_level = 5;
    @all_attacks = (\@light,
		    \@normal,
		    \@heavy,
		    \@heavyplus,
		    \@top10,
		    \@custom
		   );
}

1;
